module LotusAdmin
  module FileStreamer
    extend ActiveSupport::Concern

    included do
      include ActionController::Live

      class_attribute :_exporter_class, instance_accessor: false
      class_attribute :_exporter_filename, instance_accessor: false

      let(:exporter) { self.class._exporter_class }
    end

    # Usage
    #
    # class LotusAdmin::OrganizationsController < LotusAdmin::AuthenticatedController
    #   exporter OrganizationExporter
    #
    #   # optional filename customization in context of controller
    #   exports_filename { "organizations-#{ params[:parent_id] }" }
    # end
    class_methods do
      def exporter(exporter_class)
        self._exporter_class = exporter_class
      end

      def exports_filename(&block)
        self._exporter_filename = block
      end
    end

    # Usage
    #
    # def index
    #   super do |format|
    #     format.pdf { ... }
    #   end
    # end
    #
    # Will automatically stream CSV from configured exporter
    def index(&block)
      super do |format|
        if self.class._exporter_class.present?
          format.csv do
            if self.class._exporter_filename.present?
              filename = instance_exec(&self.class._exporter_filename)
            end

            filename = "#{ resource_class.model_name.human(count: 2) }-exports-#{ SecureRandom.uuid }" if filename.blank?

            stream_file filename.parameterize, 'csv' do |stream|
              self.class._exporter_class.stream(ransack_object.result, stream) # full query, not just paginated
            end
          end
        end

        block.call(format) if block.present?
      end
    end

    private

    def stream_file(filename, extension, &block)
      response.headers["Content-Type"] = "application/octet-stream"
      response.headers["Content-Disposition"] = "attachment; filename=#{filename}.#{extension}"

      begin
        yield response.stream
      ensure
        response.stream.close
      end
    end
  end
end
