module LotusAdmin
  class Filters::Configuration
    attr_reader :method, :resource_class

    NUMBER_PREDICATE_FILTER_OPTIONS = [['Greater Than', 'gteq'], ['Equals', 'eq'], ['Less Than', 'lteq']].freeze
    STRING_PREDICATE_FILTER_OPTIONS = [['Contains', 'cont'], ['Equals', 'eq'], ['Starts With', 'start'], ['Ends With', 'end']].freeze
    DATE_PREDICATE_FILTER_OPTIONS   = [['After', 'gteq'], ['Exactly', 'eq'], ['Before', 'lteq']].freeze

    # By default, a filter type will be selected based upon the attribute type
    #
    # For example, a date will get a datepicker, a datetime gets a datetime picker
    # A string gets a text search, and a numerical field gets a number search
    #
    # To provide a select dropdown, such as filtering on an association, simply
    # provide a collection attribute (as a Proc to be evaluated in the view context)
    #
    # Examples:
    #
    # filter :id (numeric)
    # filter :device_id (numeric)
    # filter :device_id, collection: ->{ [[1, 'device']] } (select)
    # filter :created_at (datetime)
    # filter :created_at, as: :date (date)
    # filter :name (string)

    def initialize(resource_class, method, options={})
      @resource_class = resource_class
      @method = method
      @options = options
    end

    def label
      @label ||= translate_label
    end

    def render(f)
      case as
      when :number
        render_number_input_group(f)
      when :select, :grouped_select
        render_collection(f)
      when :datetime
        render_date_or_time_input_group(f, true)
      when :date
        render_date_or_time_input_group(f, false)
      when :boolean
        render_boolean_input_group(f)
      when :string
        render_string_input_group(f)
      end
    end

    def input_options(view_ctx = nil)
      input_options = {
        collection: collection(view_ctx),
        input_html: input_html_options,
        as: as,
        label: label
      }

      input_options.delete(:collection) if input_options[:collection].nil?
      input_options.merge!(@options.fetch(:input_options, {}))

      input_options
    end

    def datepicker?
      @options.fetch(:datepicker, false)
    end

    private

    def predicate_select(f)
      input_name = "#{ method }_predicate"
      selected_value = f.template.params.fetch(input_name, nil)

      filter_options = case as
      when :number then NUMBER_PREDICATE_FILTER_OPTIONS
      when :string then STRING_PREDICATE_FILTER_OPTIONS
      when :date, :datetime then DATE_PREDICATE_FILTER_OPTIONS
      end

      f.template.select_tag("#{ method }_predicate", f.template.options_for_select(filter_options, selected: selected_value), class: 'form-control select filter-predicate-select')
    end

    def input_html_options
      options = @options.fetch(:input_html, {})
      options.merge!(class: 'datepicker', data: { field: :date }) if datepicker?

      options
    end

    def as
      # Allow for specifying a :grouped_select for instance
      unless @options[:collection].blank?
        return @options.fetch(:as, :select)
      end

      default_type = case attribute_type
      when :integer, :decimal then :number
      when :datetime then :datetime
      when :date then :date
      when :boolean then :boolean
      else
        :string
      end

      @options.fetch(:as, default_type)
    end

    def translate_label
      I18n.t(method_name.parameterize(separator: '_'), scope: "lotus_admin.filters.config.#{ resource_class.model_name.i18n_key }", default: method_name.titleize)
    end

    def collection(view_ctx)
      return if @options[:collection].nil?

      @options[:as] ||= :select

      if view_ctx.nil?
        @options[:collection].call
      else
        view_ctx.instance_exec(&@options[:collection])
      end
    end

    def method_name
      method.to_s
    end

    def attribute_type
      @attribute_type ||= resource_class.type_for_attribute(method_name).type
    end

    def render_number_input_group(f)
      render_form_group_with_predicate_and_input(f) do |value|
        f.template.text_field_tag(input_name, value, class: 'form-control string')
      end
    end

    def render_string_input_group(f)
      render_form_group_with_predicate_and_input(f) do |value|
        f.template.text_field_tag(input_name, value, class: 'form-control string')
      end
    end

    def render_collection(f)
      f.input(method, input_options(f.template))
    end

    def render_date_or_time_input_group(f, include_time = false)
      data_field_name = :date
      data_field_name = :datetime if include_time

      render_form_group_with_predicate_and_input(f) do |value|
        f.template.content_tag(:div, class: "input-group #{ data_field_name }") do
          f.template.text_field_tag(input_name, value, class: 'form-control string') +
          f.template.content_tag(:span, class: 'input-group-addon') do
            f.template.fa_icon(:calendar)
          end
        end
      end
    end

    def render_boolean_input_group(f)
      f.input("#{ method_name }_eq", as: :select, label: translate_label)
    end

    def render_form_group_with_predicate_and_input(f, &block)
      f.template.content_tag(:div, class: 'form-group', data: { attribute_name: method_name }) do
        f.template.label_tag(method, label, for: input_id) +
        f.template.content_tag(:div, class: 'row') do
          f.template.content_tag(:div, class: 'col-sm-4') do
            predicate_select(f)
          end +
          f.template.content_tag(:div, class: 'col-sm-8') do
            value = find_predicate_input_value(f)

            block.call(value)
          end
        end
      end
    end

    def find_predicate_input_value(f)
      _, value = f.template.params.fetch(:q, {}).to_unsafe_h.find do |k,_|
        k.starts_with?(method_name)
      end

      value
    end

    def input_name
      "q[#{ method_name }]"
    end

    def input_id
      ['q', method_name].join('_')
    end
  end
end
